// Require in modules
var sys = require('sys');
var rest = require('restler');
var _ = require('underscore');
var growl = require('growl');
var cron = require('cron');

// Configure script variables
var endpoints = [{
	name: 'Twitter API',
	url: 'https://api.twitter.com/1/users/show.json?screen_name=melbjs',
	complete: function(data, callback) {
		if (data.length == 0) {
			callback({ message: 'No twitter user data for melbjs', title: 'Twitter API', sticky: true });
		} else {
			callback({ message: 'All is good in the world', title: 'Twitter API', sticky: false });
		}
	},
	error: function(data, callback) {
		callback({ message: 'Error contacting API', title: 'Twitter API', sticky: true });
	}
},
{
	name: 'MelbJS EventBrite',
	url: 'http://melbjs.eventbrite.com',
	complete: function(data, callback) {
		if (data.indexOf('<h2>Wednesday, May') > 0) {
			callback({ message: 'May meetup is up!', title: 'MelbJS EventBrite', sticky: true });
		} else {
			callback({ message: 'Still April\'s meetup :(', title: 'MelbJS EventBrite', sticky: false });
		}
	},
	error: function(data, callback) {
		callback({ message: 'Error contacting EventBrite', title: 'MelbJS EventBrite', sticky: true });
	}
}];

// Loop through each endpoint and register cronjob
for (var i = 0, l = endpoints.length; i < l; i++) {
	cronify(endpoints[i]);
}

function cronify(endpoint) {
	// Register cron like job to run every minute
	sys.puts('Registering cron ' + endpoint.name);
	cron.CronJob('1 * * * * *', function() {
		sys.puts('Checking ' + endpoint.name);
		rest
			.get(endpoint.url)
			.on('complete', function(data) {
				endpoint.complete(data, notify);
			})
			.on('error', function(data) {
				endpoint.error(data, notify);
			});
	});
}

// Helper function for growl that merges options with defaults
function notify(options) {
	var defaults = {
		message: '',
		title: '',
		image: 'icon.png',
		sticky: false,
	}
	var opts = _.defaults(options, defaults);
	growl(opts.message, { title: opts.title, image: opts.image, sticky: opts.sticky });
}