#!/usr/local/bin/node

/**
 * Installation
 *
 * chmod -R 755 /Users/ben/Desktop/melbjs-talk/scripts/youtube/02/
 * sudo ln -s /Users/ben/Desktop/melbjs-talk/scripts/youtube/02/app.js /usr/local/bin/youtube
 *
 * Examples
 *
 * youtube QH2-TGUlwu4
 * youtube QH2-TGUlwu4 /Users/ben/Desktop
 * youtube QH2-TGUlwu4 /Users/ben/Desktop high
 *
 */

// Require in modules
var youtube = require('youtube-dl');

// Get command line args
var args = process.argv.slice(2); // slice off 1st 2 items
var videoId = args[0] || 'QH2-TGUlwu4';
var output = args[1] || '/Users/ben/Desktop';
var quality = args[2] || 'low';

console.log('Command Line Args:');
console.log('videoId:', videoId);
console.log('output:', output);
console.log('quality:', quality);

// Define formats lookup
var formats = {
	'1080p': '37',
	'720p': '22',
	'high': '35',
	'low': '34',
	'mobile': '17',
}

// Use passed in quality to determine format
var format = formats[quality] || formats['low'];

// Construct script variables
var path = 'http://www.youtube.com/watch?v=' + videoId;

// Get video clip information
youtube.info(path, function(err, info) {
	if (err) { throw err; }
	console.log(info);
});

// Begin downloading video clip
var download = youtube.download(path, output, ['--max-quality=18', '--format=' + format]);

// On download starting
download.on('download', function(data) {
  console.log('Download started');
  console.log('filename: ' + data.filename);
  console.log('size: ' + data.size);
});

// As download progresses
download.on('progress', function(data) {
	process.stdout.write(data.eta + ' ' + data.percent + '% at ' + data.speed + '\r');
});

// Catches any errors
download.on('error', function(err) {
	throw err;
});

// On download finishing
download.on('end', function(data) {
	console.log('\nDownload finished!');
	console.log('Filename: ' + data.filename);
	console.log('Size: ' + data.size);
	console.log('Time Taken: ' + data.timeTaken);
	console.log('Time Taken in ms: ' + data.timeTakenms);
	console.log('Average Speed: ' + data.averageSpeed);
	console.log('Average Speed in Bytes: ' + data.averageSpeedBytes);
});