#!/usr/local/bin/node

/**
 * Installation
 *
 * chmod -R 755 /Users/ben/Desktop/melbjs-talk/scripts/youtube/03/
 * sudo ln -s /Users/ben/Desktop/melbjs-talk/scripts/youtube/03/app.js /usr/local/bin/youtube
 *
 * Examples
 *
 * youtube -v QH2-TGUlwu4
 * youtube -v QH2-TGUlwu4 -d /Users/ben/Desktop
 * youtube -v QH2-TGUlwu4 -d /Users/ben/Desktop -q high
 *
 */

// Require in modules
var youtube = require('youtube-dl');
var commander = require('commander');

// Get command line args
commander
	.version('0.0.1')
	.option('-v, --video <id>', 'Video id of youtube clip')
	.option('-d, --dest <filepath>', 'Destination directory [/Users/ben/Desktop]', '/Users/ben/Desktop')
	.option('-q, --quality <level>', 'Quality level (1080p|720p|high|low|mobile) [low]', 'low')
	.parse(process.argv);

var videoId = commander.video;
var output = commander.dest;
var quality = commander.quality;

console.log('Command Line Args:');
console.log('videoId:', videoId);
console.log('output:', output);
console.log('quality:', quality);

if (typeof videoId == 'undefined') {
	console.log('ERROR: Please specify valid video id');
	return;
}

// Define formats lookup
var formats = {
	'1080p': '37',
	'720p': '22',
	'high': '35',
	'low': '34',
	'mobile': '17',
};

// Use passed in quality to determine format
var format = formats[quality] || formats['low'];

// Construct script variables
var path = 'http://www.youtube.com/watch?v=' + videoId;

// Get video clip information
youtube.info(path, function(err, info) {
	if (err) { throw err; }
	console.log(info);
});

// Begin downloading video clip
var download = youtube.download(path, output, ['--max-quality=18', '--format=' + format]);

// On download starting
download.on('download', function(data) {
  console.log('Download started');
  console.log('filename: ' + data.filename);
  console.log('size: ' + data.size);
});

// As download progresses
download.on('progress', function(data) {
	process.stdout.write(data.eta + ' ' + data.percent + '% at ' + data.speed + '\r');
});

// Catches any errors
download.on('error', function(err) {
	throw err;
});

// On download finishing
download.on('end', function(data) {
	console.log('\nDownload finished!');
	console.log('Filename: ' + data.filename);
	console.log('Size: ' + data.size);
	console.log('Time Taken: ' + data.timeTaken);
	console.log('Time Taken in ms: ' + data.timeTakenms);
	console.log('Average Speed: ' + data.averageSpeed);
	console.log('Average Speed in Bytes: ' + data.averageSpeedBytes);
});